package com.miaxis.film.controller;

import com.miaxis.common.constant.Constants;
import java.util.List;
import java.util.Arrays;
import io.swagger.annotations.*;
import com.miaxis.common.core.domain.Response;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.ModelAttribute;
import com.miaxis.common.annotation.Log;
import com.miaxis.common.core.controller.BaseController;
import com.miaxis.common.enums.BusinessTypeEnum;
import com.miaxis.film.domain.RefundRecord;
import com.miaxis.film.service.IRefundRecordService;
import com.miaxis.common.utils.poi.ExcelUtil;
import com.miaxis.common.core.page.ResponsePageInfo;

/**
 * 【微信退款记录】Controller
 *
 * @author miaxis
 * @date 2021-05-18
 */
@RestController
@RequestMapping("/film/record")
@Api(tags={"【小程序-微信退款记录】"})
public class RefundRecordController extends BaseController{
    @Autowired
    private IRefundRecordService refundRecordService;

    /**
     * 查询微信退款记录列表
     */
    @PreAuthorize("@ss.hasPermi('film:record:list')")
    @GetMapping("/list")
    @ApiOperation("查询微信退款记录列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum",value = "当前页码" ,dataType = "int", paramType = "query", required = false),
            @ApiImplicitParam(name = "pageSize",value = "每页数据量" , dataType = "int", paramType = "query", required = false),
    })
    public ResponsePageInfo<RefundRecord> list(@ModelAttribute RefundRecord refundRecord){
        startPage();
        List<RefundRecord> list = refundRecordService.selectRefundRecordList(refundRecord);
        return toResponsePageInfo(list);
    }

    /**
     * 导出微信退款记录列表
     */
    @PreAuthorize("@ss.hasPermi('film:record:export')")
    @Log(title = "微信退款记录", businessType = BusinessTypeEnum.EXPORT)
    @GetMapping("/export")
    @ApiOperation("导出微信退款记录列表Excel")
    public Response<String> export(@ModelAttribute RefundRecord refundRecord){
        List<RefundRecord> list = refundRecordService.selectRefundRecordList(refundRecord);
        ExcelUtil<RefundRecord> util = new ExcelUtil<RefundRecord>(RefundRecord.class);
        return util.exportExcel(list, "record");
    }

    /**
     * 获取微信退款记录详细信息
     */
    @PreAuthorize("@ss.hasPermi('film:record:query')")
    @GetMapping(value = "/{id}")
    @ApiOperation("获取微信退款记录详细信息")
    public Response<RefundRecord> getInfo(
            @ApiParam(name = "id", value = "微信退款记录参数", required = true)
            @PathVariable("id") Long id
    ){
        return Response.success(refundRecordService.getById(id));
    }

    /**
     * 新增微信退款记录
     */
    @PreAuthorize("@ss.hasPermi('film:record:add')")
    @Log(title = "微信退款记录", businessType = BusinessTypeEnum.INSERT)
    @PostMapping
    @ApiOperation("新增微信退款记录")
    public Response<Integer> add(@RequestBody RefundRecord refundRecord){
        return toResponse(refundRecordService.save(refundRecord) ? 1 : 0);
    }

    /**
     * 修改微信退款记录
     */
    @PreAuthorize("@ss.hasPermi('film:record:edit')")
    @Log(title = "微信退款记录", businessType = BusinessTypeEnum.UPDATE)
    @PutMapping
    @ApiOperation("修改微信退款记录")
    public Response<Integer> edit(@RequestBody RefundRecord refundRecord){
        return toResponse(refundRecordService.updateById(refundRecord) ? 1 : 0);
    }

    /**
     * 删除微信退款记录
     */
    @PreAuthorize("@ss.hasPermi('film:record:remove')")
    @Log(title = "微信退款记录", businessType = BusinessTypeEnum.DELETE)
	@DeleteMapping("/{ids}")
    @ApiOperation("删除微信退款记录")
    public  Response<Integer> remove(
            @ApiParam(name = "ids", value = "微信退款记录ids参数", required = true)
            @PathVariable Long[] ids
    ){
        return toResponse(refundRecordService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
