package com.miaxis.film.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.miaxis.common.config.WxpayConfig;
import com.miaxis.common.exception.CustomException;
import com.miaxis.film.domain.FilmOrder;
import com.miaxis.film.domain.RefundRecord;
import com.miaxis.film.dto.WxNotifyReturnDTO;
import com.miaxis.film.mapper.RefundRecordMapper;
import com.miaxis.film.service.IRefundRecordService;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.util.EntityUtils;
import org.joda.time.DateTime;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.ByteArrayOutputStream;
import java.util.List;

/**
 * 微信退款记录Service业务层处理
 *
 * @author miaxis
 * @date 2021-05-18
 */
@Service
public class RefundRecordServiceImpl extends ServiceImpl<RefundRecordMapper, RefundRecord> implements IRefundRecordService {
    @Resource
    private RefundRecordMapper refundRecordMapper;

    @Autowired
    private HttpClient httpClient;


    @Autowired
    private WxpayConfig wxpayConfig;
    /**
     * 查询微信退款记录列表
     *
     * @param refundRecord 微信退款记录
     * @return 微信退款记录
     */
    @Override
    public List<RefundRecord> selectRefundRecordList(RefundRecord refundRecord){
        return refundRecordMapper.selectRefundRecordList(refundRecord);
    }

    @Override
    public WxNotifyReturnDTO refund(FilmOrder filmOrder, String refundCode) throws Exception{
        HttpPost httpPost = new HttpPost(wxpayConfig.getV3urlRefund());
        httpPost.addHeader("Accept", "application/json");
        httpPost.addHeader("Content-type","application/json; charset=utf-8");
        ObjectMapper objectMapper = new ObjectMapper();
        ObjectNode rootNode = objectMapper.createObjectNode();

        rootNode.put("out_trade_no", filmOrder.getOutTradeNo())
                .put("out_refund_no", refundCode)
                .put("reason", "电影购票失败")
                .put("notify_url",wxpayConfig.getNotifyUrlRefund());
        rootNode.putObject("amount")
                .put("refund", filmOrder.getTotal())
                .put("total", filmOrder.getTotal())
                .put("currency", filmOrder.getCurrency());
        System.out.println(rootNode.toString());
//        ByteArrayOutputStream bos = new ByteArrayOutputStream();
//        objectMapper.writeValue(bos, rootNode);
        httpPost.setEntity(new StringEntity(rootNode.toString(), "utf-8"));
        HttpResponse response = httpClient.execute(httpPost);
        // 返回数据：
        String bodyAsString = EntityUtils.toString(response.getEntity());
        if(response.getStatusLine().getStatusCode() == HttpStatus.SC_OK){
            //  保存退款数据到数据库
            RefundRecord refundRecord = new RefundRecord();
            JSONObject jsonObject = JSONObject.parseObject(bodyAsString);
            refundRecord.setOutRefundNo(jsonObject.getString("refund_id"));
            refundRecord.setTransactionId(jsonObject.getString("out_refund_no"));
            refundRecord.setOutTradeNo(jsonObject.getString("out_trade_no"));
            refundRecord.setChannel(jsonObject.getString("channel"));
            refundRecord.setUserReceivedAccount(jsonObject.getString("user_received_account"));
            if (jsonObject.getString("success_time")!=null){
                refundRecord.setSuccessTime(new DateTime(jsonObject.getString("success_time")).toDate());
            }
            refundRecord.setCreateTime(new DateTime(jsonObject.getString("create_time")).toDate());
            refundRecord.setStatus(jsonObject.getString("status"));
            refundRecord.setFundsAccount(jsonObject.getString("funds_account"));
            JSONObject amount = jsonObject.getJSONObject("amount");
            refundRecord.setTotal(amount.getInteger("total"));
            refundRecord.setRefund(amount.getInteger("refund"));
            refundRecord.setPayerTotal(amount.getInteger("payer_total"));
            refundRecord.setPayerRefund(amount.getInteger("payer_refund"));
            refundRecord.setSettlementRefund(amount.getInteger("settlement_refund"));
            refundRecord.setSettlementTotal(amount.getInteger("settlement_total"));
            refundRecord.setDiscountRefund(amount.getInteger("discount_refund"));
            refundRecord.setCurrency(amount.getString("currency"));
            this.save(refundRecord);

            WxNotifyReturnDTO wxNotifyReturnDTO = new WxNotifyReturnDTO();
            wxNotifyReturnDTO.setCode("SUCCESS");
            wxNotifyReturnDTO.setMessage("成功");
            return wxNotifyReturnDTO;
        }else{
            throw new CustomException(bodyAsString);
        }

    }

    @Override
    public RefundRecord getByRefundId(String refundId) {
        return refundRecordMapper.getByRefundId(refundId);
    }
}
